/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.envmonitor.historywarn.service.impl;

import me.zhengjie.modules.dm.envmonitor.historywarn.domain.DmEnvHistoryWarn;
import me.zhengjie.utils.ValidationUtil;
import me.zhengjie.utils.FileUtil;
import lombok.RequiredArgsConstructor;
import me.zhengjie.modules.dm.envmonitor.historywarn.repository.DmEnvHistoryWarnRepository;
import me.zhengjie.modules.dm.envmonitor.historywarn.service.DmEnvHistoryWarnService;
import me.zhengjie.modules.dm.envmonitor.historywarn.service.dto.DmEnvHistoryWarnDto;
import me.zhengjie.modules.dm.envmonitor.historywarn.service.dto.DmEnvHistoryWarnQueryCriteria;
import me.zhengjie.modules.dm.envmonitor.historywarn.service.mapstruct.DmEnvHistoryWarnMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import me.zhengjie.utils.PageUtil;
import me.zhengjie.utils.QueryHelp;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.LinkedHashMap;

/**
* @website https://el-admin.vip
* @description 服务实现
* @author lr
* @date 2022-05-19
**/
@Service
@RequiredArgsConstructor
public class DmEnvHistoryWarnServiceImpl implements DmEnvHistoryWarnService {

    private final DmEnvHistoryWarnRepository dmEnvHistoryWarnRepository;
    private final DmEnvHistoryWarnMapper dmEnvHistoryWarnMapper;

    @Override
    public Map<String,Object> queryAll(DmEnvHistoryWarnQueryCriteria criteria, Pageable pageable){
        Page<DmEnvHistoryWarn> page = dmEnvHistoryWarnRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder),pageable);
        return PageUtil.toPage(page.map(dmEnvHistoryWarnMapper::toDto));
    }

    @Override
    public List<DmEnvHistoryWarnDto> queryAll(DmEnvHistoryWarnQueryCriteria criteria){
        return dmEnvHistoryWarnMapper.toDto(dmEnvHistoryWarnRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder)));
    }

    @Override
    @Transactional
    public DmEnvHistoryWarnDto findById(Long id) {
        DmEnvHistoryWarn dmEnvHistoryWarn = dmEnvHistoryWarnRepository.findById(id).orElseGet(DmEnvHistoryWarn::new);
        ValidationUtil.isNull(dmEnvHistoryWarn.getId(),"DmEnvHistoryWarn","id",id);
        return dmEnvHistoryWarnMapper.toDto(dmEnvHistoryWarn);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DmEnvHistoryWarnDto create(DmEnvHistoryWarn resources) {
        return dmEnvHistoryWarnMapper.toDto(dmEnvHistoryWarnRepository.save(resources));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DmEnvHistoryWarn resources) {
        DmEnvHistoryWarn dmEnvHistoryWarn = dmEnvHistoryWarnRepository.findById(resources.getId()).orElseGet(DmEnvHistoryWarn::new);
        ValidationUtil.isNull( dmEnvHistoryWarn.getId(),"DmEnvHistoryWarn","id",resources.getId());
        dmEnvHistoryWarn.copy(resources);
        dmEnvHistoryWarnRepository.save(dmEnvHistoryWarn);
    }

    @Override
    public void deleteAll(Long[] ids) {
        for (Long id : ids) {
            dmEnvHistoryWarnRepository.deleteById(id);
        }
    }

    @Override
    public void download(List<DmEnvHistoryWarnDto> all, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (DmEnvHistoryWarnDto dmEnvHistoryWarn : all) {
            Map<String,Object> map = new LinkedHashMap<>();
            map.put("设备编号", dmEnvHistoryWarn.getDeviceIdentifier());
            map.put("设备名称", dmEnvHistoryWarn.getDeviceName());
            map.put("创建人", dmEnvHistoryWarn.getCreateBy());
            map.put("更新人", dmEnvHistoryWarn.getUpdateBy());
            map.put("创建时间", dmEnvHistoryWarn.getCreateTime());
            map.put("更新时间", dmEnvHistoryWarn.getUpdateTime());
            map.put("告警指标", dmEnvHistoryWarn.getWarnIndex());
            map.put("告警值", dmEnvHistoryWarn.getWarnValue());
            map.put("告警内容", dmEnvHistoryWarn.getWarnContent());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }
}