/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.envmonitor.peptdevicenew.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.envmonitor.peptdevicenew.domain.DmEnvPeptDeciveNew;
import me.zhengjie.modules.dm.envmonitor.peptdevicenew.service.DmEnvPeptDeciveNewService;
import me.zhengjie.modules.dm.envmonitor.peptdevicenew.service.dto.DmEnvPeptDeciveNewQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author wanglongda
* @date 2022-05-13
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "感知设备管理")
@RequestMapping("/api/dmEnvPeptDeciveNew")
public class DmEnvPeptDeciveNewController {

    private final DmEnvPeptDeciveNewService dmEnvPeptDeciveNewService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmEnvPeptDeciveNew:list')")
    public void download(HttpServletResponse response, DmEnvPeptDeciveNewQueryCriteria criteria) throws IOException {
        dmEnvPeptDeciveNewService.download(dmEnvPeptDeciveNewService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询感知设备")
    @ApiOperation("查询感知设备")
    @PreAuthorize("@el.check('dmEnvPeptDeciveNew:list')")
    public ResponseEntity<Object> query(DmEnvPeptDeciveNewQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmEnvPeptDeciveNewService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增感知设备")
    @ApiOperation("新增感知设备")
    @PreAuthorize("@el.check('dmEnvPeptDeciveNew:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmEnvPeptDeciveNew resources){
        return new ResponseEntity<>(dmEnvPeptDeciveNewService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改感知设备")
    @ApiOperation("修改感知设备")
    @PreAuthorize("@el.check('dmEnvPeptDeciveNew:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmEnvPeptDeciveNew resources){
        dmEnvPeptDeciveNewService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除感知设备")
    @ApiOperation("删除感知设备")
    @PreAuthorize("@el.check('dmEnvPeptDeciveNew:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmEnvPeptDeciveNewService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}