/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.modbus.data.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.modbus.data.domain.DmModbusData;
import me.zhengjie.modules.dm.modbus.data.service.DmModbusDataService;
import me.zhengjie.modules.dm.modbus.data.service.dto.DmModbusDataQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author wld
* @date 2022-05-21
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "分组下的数据管理管理")
@RequestMapping("/api/dmModbusData")
public class DmModbusDataController {

    private final DmModbusDataService dmModbusDataService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmModbusData:list')")
    public void download(HttpServletResponse response, DmModbusDataQueryCriteria criteria) throws IOException {
        dmModbusDataService.download(dmModbusDataService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询分组下的数据管理")
    @ApiOperation("查询分组下的数据管理")
    @PreAuthorize("@el.check('dmModbusData:list')")
    public ResponseEntity<Object> query(DmModbusDataQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmModbusDataService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增分组下的数据管理")
    @ApiOperation("新增分组下的数据管理")
    @PreAuthorize("@el.check('dmModbusData:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmModbusData resources){
        return new ResponseEntity<>(dmModbusDataService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改分组下的数据管理")
    @ApiOperation("修改分组下的数据管理")
    @PreAuthorize("@el.check('dmModbusData:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmModbusData resources){
        dmModbusDataService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除分组下的数据管理")
    @ApiOperation("删除分组下的数据管理")
    @PreAuthorize("@el.check('dmModbusData:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmModbusDataService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}