/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.modbus.device.service;

import me.zhengjie.modules.dm.modbus.device.domain.DmModbusDevice;
import me.zhengjie.modules.dm.modbus.device.service.dto.DmModbusDeviceDto;
import me.zhengjie.modules.dm.modbus.device.service.dto.DmModbusDeviceQueryCriteria;
import org.springframework.data.domain.Pageable;

import java.sql.Timestamp;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author wld
* @date 2022-05-20
**/
public interface DmModbusDeviceService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmModbusDeviceQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmModbusDeviceDto>
    */
    List<DmModbusDeviceDto> queryAll(DmModbusDeviceQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param deviceId ID
     * @return DmModbusDeviceDto
     */
    DmModbusDeviceDto findById(Long deviceId);

    /**
    * 创建
    * @param resources /
    * @return DmModbusDeviceDto
    */
    DmModbusDeviceDto create(DmModbusDevice resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmModbusDevice resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmModbusDeviceDto> all, HttpServletResponse response) throws IOException;

    Long getModbusCount(Integer status);

    int updateDeviceStatus(Timestamp timestamp, Long deviceId, String status);
}