/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.modbus.deviceStatus.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.modbus.deviceStatus.domain.DmModbusDeviceStatus;
import me.zhengjie.modules.dm.modbus.deviceStatus.service.DmModbusDeviceStatusService;
import me.zhengjie.modules.dm.modbus.deviceStatus.service.dto.DmModbusDeviceStatusQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author wld
* @date 2022-05-26
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "设备在线日志管理管理")
@RequestMapping("/api/dmModbusDeviceStatus")
public class DmModbusDeviceStatusController {

    private final DmModbusDeviceStatusService dmModbusDeviceStatusService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
//    @PreAuthorize("@el.check('dmModbusDeviceStatus:list')")
    public void download(HttpServletResponse response, DmModbusDeviceStatusQueryCriteria criteria) throws IOException {
        dmModbusDeviceStatusService.download(dmModbusDeviceStatusService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询设备在线日志管理")
    @ApiOperation("查询设备在线日志管理")
//    @PreAuthorize("@el.check('dmModbusDeviceStatus:list')")
    public ResponseEntity<Object> query(DmModbusDeviceStatusQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmModbusDeviceStatusService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增设备在线日志管理")
    @ApiOperation("新增设备在线日志管理")
//    @PreAuthorize("@el.check('dmModbusDeviceStatus:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmModbusDeviceStatus resources){
        return new ResponseEntity<>(dmModbusDeviceStatusService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改设备在线日志管理")
    @ApiOperation("修改设备在线日志管理")
//    @PreAuthorize("@el.check('dmModbusDeviceStatus:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmModbusDeviceStatus resources){
        dmModbusDeviceStatusService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除设备在线日志管理")
    @ApiOperation("删除设备在线日志管理")
//    @PreAuthorize("@el.check('dmModbusDeviceStatus:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmModbusDeviceStatusService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}