/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.modbus.drive.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.modbus.drive.domain.DmModbusDrive;
import me.zhengjie.modules.dm.modbus.drive.service.DmModbusDriveService;
import me.zhengjie.modules.dm.modbus.drive.service.dto.DmModbusDriveQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author wld
* @date 2022-05-20
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "modbus驱动管理管理")
@RequestMapping("/api/dmModbusDrive")
public class DmModbusDriveController {

    private final DmModbusDriveService dmModbusDriveService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmModbusDrive:list')")
    public void download(HttpServletResponse response, DmModbusDriveQueryCriteria criteria) throws IOException {
        dmModbusDriveService.download(dmModbusDriveService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询modbus驱动管理")
    @ApiOperation("查询modbus驱动管理")
    @PreAuthorize("@el.check('dmModbusDrive:list')")
    public ResponseEntity<Object> query(DmModbusDriveQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmModbusDriveService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增modbus驱动管理")
    @ApiOperation("新增modbus驱动管理")
    @PreAuthorize("@el.check('dmModbusDrive:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmModbusDrive resources){
        return new ResponseEntity<>(dmModbusDriveService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改modbus驱动管理")
    @ApiOperation("修改modbus驱动管理")
    @PreAuthorize("@el.check('dmModbusDrive:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmModbusDrive resources){
        dmModbusDriveService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除modbus驱动管理")
    @ApiOperation("删除modbus驱动管理")
    @PreAuthorize("@el.check('dmModbusDrive:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmModbusDriveService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}