/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.modbus.group.service;

import me.zhengjie.modules.dm.modbus.group.domain.DmModbusGroup;
import me.zhengjie.modules.dm.modbus.group.service.dto.DmModbusGroupDto;
import me.zhengjie.modules.dm.modbus.group.service.dto.DmModbusGroupQueryCriteria;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author wld
* @date 2022-05-21
**/
public interface DmModbusGroupService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmModbusGroupQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmModbusGroupDto>
    */
    List<DmModbusGroupDto> queryAll(DmModbusGroupQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param groupId ID
     * @return DmModbusGroupDto
     */
    DmModbusGroupDto findById(Long groupId);

    /**
    * 创建
    * @param resources /
    * @return DmModbusGroupDto
    */
    DmModbusGroupDto create(DmModbusGroup resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmModbusGroup resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmModbusGroupDto> all, HttpServletResponse response) throws IOException;
}